----------------------------------------------------------------
--  Title         : SCC_MAP.vhd
--  Original file : PSGSCCR from Kazuhiro TSUJIKAWA (ESE Artists' factory)
--  Modification  : SCC/PSG sound removed (FABF 2013)
--  Function      : SCC MAPPER (EseSCC/SNATCHER compatible)
----------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

entity SCC_MAP is
  port(
    pSltClk     : IN std_logic;
    pSltRst_n   : IN std_logic;
    pSltSltsl_n : IN std_logic;
    pSltIorq_n  : IN std_logic;
    pSltRd_n    : IN std_logic;
    pSltWr_n    : IN std_logic;
    pSltAdr     : IN std_logic_vector(15 downto 0);
    pSltDat     : IN std_logic_vector(7 downto 0);
    pSltBdir_n  : OUT std_logic;

    pRamCeX_n   : OUT std_logic;
    pRamOeX_n   : OUT std_logic;
    pRamWeX_n   : OUT std_logic;
    pRamAdrX    : OUT std_logic_vector(18 downto 0);
    pRamDatX    : INOUT std_logic_vector(7 downto 0)

 );
end SCC_MAP;

architecture RTL of SCC_MAP is

  signal pSltClk_n   : std_logic;
  signal DevHit      : std_logic;
  signal Dec1FFE     : std_logic;
  signal DecSccA     : std_logic;
  signal DecSccB     : std_logic;

  signal SccBank0    : std_logic_vector(7 downto 0);
  signal SccBank1    : std_logic_vector(7 downto 0);
  signal SccBank2    : std_logic_vector(7 downto 0);
  signal SccBank3    : std_logic_vector(7 downto 0);
  signal SccModeA    : std_logic_vector(7 downto 0);
  signal SccModeB    : std_logic_vector(7 downto 0);

  signal SccRegWe    : std_logic;
  signal LSccRegWe   : std_logic;
  signal RSccRegWe   : std_logic;

  signal SccModWe    : std_logic;
  signal LSccModWe   : std_logic;
  signal RSccModWe   : std_logic;

  signal SccWavCe    : std_logic;
  signal LSccWavCe   : std_logic;
  signal RSccWavCe   : std_logic;

  signal SccWavOe    : std_logic;
  signal LSccWavOe   : std_logic;
  signal RSccWavOe   : std_logic;

  signal SccWavWe    : std_logic;
  signal LSccWavWe   : std_logic;
  signal RSccWavWe   : std_logic;

  signal SccWavWx    : std_logic;
  signal LSccWavWx   : std_logic;
  signal RSccWavWx   : std_logic;
  signal SccWavAdr   : std_logic_vector(4 downto 0);
  signal SccWavDat   : std_logic_vector(7 downto 0);

  signal SccMemCs    : std_logic;
  signal RamAdrEx    : std_logic;
  signal SccSpcCs    : std_logic;
  signal SccDmack    : std_logic;

  
begin

  ----------------------------------------------------------------
  -- Dummy pin
  ----------------------------------------------------------------
 
  pSltClk_n <= not pSltClk;

  pSltBdir_n <= '0' when pSltSltsl_n = '0' and pSltRd_n = '0' else '1';

  ----------------------------------------------------------------
  -- Slot access control
  ----------------------------------------------------------------
  process(pSltClk_n, pSltRst_n, pSltIorq_n, pSltSltsl_n, pSltRd_n, pSltWr_n)

    variable DevAcs0 : std_logic;
    variable DevAcs1 : std_logic;

  begin

    if ((pSltIorq_n = '0' or pSltSltsl_n = '0') and (pSltRd_n = '0' or pSltWr_n = '0')) then
      DevAcs0 := '1';
    else
      DevAcs0 := '0';
    end if;

    if (DevAcs0 = '1' and DevAcs1 = '0') then
      DevHit <= '1';
    else
      DevHit <= '0';
    end if;

    if (pSltRst_n = '0') then
      DevAcs1 := '0';
    elsif (pSltClk_n'event and pSltClk_n = '1') then
      DevAcs1 := DevAcs0;
    end if;

  end process;

  Dec1FFE <= '1' when pSltAdr(12 downto 1) = "111111111111" else '0';
  DecSccA <= '1' when pSltAdr(15 downto 11) = "10011" and SccModeB(5) = '0' and SccBank2(5 downto 0) = "111111"
                 else '0';
  DecSccB <= '1' when pSltAdr(15 downto 11) = "10111" and SccModeB(5) = '1' and SccBank3(7) = '1'
                 else '0';

  ----------------------------------------------------------------
  -- SCC register / wave memory access
  ----------------------------------------------------------------
  process(pSltClk_n, pSltRst_n)

  begin

    if (pSltRst_n = '0') then

      SccBank0   <= "00000000";
      SccBank1   <= "00000001";
      SccBank2   <= "00000010";
      SccBank3   <= "00000011";
      SccModeA   <= (others => '0');
      SccModeB   <= (others => '0');

      SccWavWx   <= '0';
      LSccWavWx  <= '0';
      RSccWavWx  <= '0';
      SccWavAdr  <= (others => '0');
      SccWavDat  <= (others => '0');

    elsif (pSltClk_n'event and pSltClk_n = '1') then

      -- Mapped I/O port access on 5000-57FFh ... Bank resister write
      if (pSltSltsl_n = '0' and pSltWr_n = '0' and pSltAdr(15 downto 11) = "01010" and
          SccModeA(6) = '0' and SccModeA(4) = '0' and SccModeB(4) = '0') then
        SccBank0 <= pSltDat;
      end if;
      -- Mapped I/O port access on 7000-77FFh ... Bank resister write
      if (pSltSltsl_n = '0' and pSltWr_n = '0' and pSltAdr(15 downto 11) = "01110" and
          SccModeA(6) = '0' and SccModeA(4) = '0' and SccModeB(4) = '0') then
        SccBank1 <= pSltDat;
      end if;
      -- Mapped I/O port access on 9000-97FFh ... Bank resister write
      if (pSltSltsl_n = '0' and pSltWr_n = '0' and pSltAdr(15 downto 11) = "10010" and
          SccModeB(4) = '0') then
        SccBank2 <= pSltDat;
      end if;
      -- Mapped I/O port access on B000-B7FFh ... Bank resister write
      if (pSltSltsl_n = '0' and pSltWr_n = '0' and pSltAdr(15 downto 11) = "10110" and
          SccModeA(6) = '0' and SccModeA(4) = '0' and SccModeB(4) = '0') then
        SccBank3 <= pSltDat;
      end if;

      -- Mapped I/O port access on 7FFE-7FFFh ... Resister write
      if (pSltSltsl_n = '0' and pSltWr_n = '0' and pSltAdr(15 downto 13) = "011" and Dec1FFE = '1' and
          SccModeB(5 downto 4) = "00") then
        SccModeA <= pSltDat;
      end if;

      -- Mapped I/O port access on BFFE-BFFFh ... Resister write
      if (pSltSltsl_n = '0' and pSltWr_n = '0' and pSltAdr(15 downto 13) = "101" and Dec1FFE = '1' and
          SccModeA(6) = '0' and SccModeA(4) = '0') then
        SccModeB <= pSltDat;
      end if;

      -- Mapped I/O port access on 9860-987Fh ... Wave memory copy
      if (pSltSltsl_n = '0' and pSltWr_n = '0' and pSltAdr(7 downto 5) = "011" and
          DevHit = '1' and SccModeB(4) = '0' and DecSccA = '1') then
        SccWavAdr <= pSltAdr(4 downto 0);
        SccWavDat <= pSltDat;
        SccWavWx  <= '1';
        if (pSltAdr(9 downto 8) /= "11") then LSccWavWx <= '1'; end if;
        if (pSltAdr(9 downto 8) /= "10") then RSccWavWx <= '1'; end if;
      else
        SccWavWx  <= '0';
        LSccWavWx <= '0';
        RSccWavWx <= '0';
      end if;

    end if;

  end process;

  -- Mapped I/O port access on 9800-987Fh / B800-B89Fh ... Wave memory
  SccWavCe <= '1' when pSltSltsl_n = '0' and DevHit = '1' and SccModeB(4) = '0' and
                       (DecSccA = '1' or DecSccB = '1')
                  else '0';

  LSccWavCe <= '1' when SccWavCe = '1' else '0';
  RSccWavCe <= '1' when SccWavCe = '1' else '0';

  -- Mapped I/O port access on 9800-987Fh / B800-B89Fh ... Wave memory
  SccWavOe <= '1' when pSltSltsl_n = '0' and pSltRd_n = '0' and SccModeB(4) = '0' and
                       ((DecSccA = '1' and pSltAdr(7) = '0') or
                        (DecSccB = '1' and (pSltAdr(7) = '0' or pSltAdr(6 downto 5) = "00")))
                  else '0';

  LSccWavOe <= '1' when SccWavOe = '1' and pSltAdr(8) = '0' else '0';
  RSccWavOe <= '1' when SccWavOe = '1' and pSltAdr(8) = '1' else '0';

  -- Mapped I/O port access on 9800-987Fh / B800-B89Fh ... Wave memory
  SccWavWe <= '1' when pSltSltsl_n = '0' and pSltWr_n = '0' and DevHit = '1' and SccModeB(4) = '0' and
                       ((DecSccA = '1' and pSltAdr(7) = '0') or DecSccB = '1')
                  else '0';

  LSccWavWe <= '1' when SccWavWe = '1' and pSltAdr(9 downto 8) /= "11" else '0';
  RSccWavWe <= '1' when SccWavWe = '1' and pSltAdr(9 downto 8) /= "10" else '0';

  -- Mapped I/O port access on 9880-988Fh / B8A0-B8AF ... Resister write
  SccRegWe <= '1' when pSltSltsl_n = '0' and pSltWr_n = '0' and
                       ((DecSccA = '1' and pSltAdr(7 downto 5) = "100") or
                        (DecSccB = '1' and pSltAdr(7 downto 5) = "101")) and
                       DevHit = '1' and SccModeB(4) = '0'
                  else '0';

  LSccRegWe <= '1' when SccRegWe = '1' and pSltAdr(9 downto 8) /= "11" else '0';
  RSccRegWe <= '1' when SccRegWe = '1' and pSltAdr(9 downto 8) /= "10" else '0';

  -- Mapped I/O port access on 98C0-98FFh / B8C0-B8DFh ... Resister write
  SccModWe <= '1' when pSltSltsl_n = '0' and pSltWr_n = '0' and pSltAdr(7 downto 6) = "11" and
                       (DecSccA = '1' or (pSltAdr(5) = '0' and DecSccB = '1')) and
                       DevHit = '1' and SccModeB(4) = '0'
                  else '0';

  LSccModWe <= '1' when SccModWe = '1' and pSltAdr(9 downto 8) /= "11" else '0';
  RSccModWe <= '1' when SccModWe = '1' and pSltAdr(9 downto 8) /= "10" else '0';

  ----------------------------------------------------------------
  -- SPC / External memory (512kbytes * 2) access
  ----------------------------------------------------------------

  -- SCSI controller access enable (4000-5FFFh/A000-BFFFh)
  SccSpcCs <= '1' when pSltSltsl_n = '0' and pSltRst_n = '1' and SccModeA(6) = '1' and SccModeA(4) = '0' and
                       (pSltAdr(15 downto 12) = "0101" or pSltAdr(15 downto 12) = "1011") else '0';
  SccDmack <= '1' when pSltSltsl_n = '0' and pSltRst_n = '1' and SccModeA(6) = '1' and SccModeA(4) = '0' and
                       (pSltAdr(15 downto 12) = "0100" or pSltAdr(15 downto 12) = "1010") else '0';

  -- Memory access enable
  SccMemCs <= '1' when pSltSltsl_n = '0' and pSltRst_n = '1' and (
                       -- 4000-5FFFh(R)
                       (pSltAdr(15 downto 13) = "010" and SccModeA(6) = '0' and                   pSltRd_n = '0') or
                       -- 6000-7FFFh(R)
                       (pSltAdr(15 downto 13) = "011" and                                         pSltRd_n = '0') or
                       -- 8000-9FFFh(R)
                       (pSltAdr(15 downto 13) = "100" and                       DecSccA = '0' and pSltRd_n = '0') or
                       -- A000-BFFFh(R)
                       (pSltAdr(15 downto 13) = "101" and SccModeA(6) = '0' and DecSccB = '0' and pSltRd_n = '0') or
                       -- 4000-5FFFh(R/W) ESCC-RAM
                       (pSltAdr(15 downto 13) = "010" and SccModeA(4) = '1') or
                       -- 6000-7FFDh(R/W) ESCC-RAM
                       (pSltAdr(15 downto 13) = "011" and SccModeA(4) = '1' and Dec1FFE /= '1') or
                       -- 4000-7FFFh(R/W) SNATCHER
                       (pSltAdr(15 downto 14) = "01"  and SccModeB(4) = '1') or
                       -- 8000-9FFFh(R/W) SNATCHER
                       (pSltAdr(15 downto 13) = "100" and SccModeB(4) = '1') or
                       -- A000-BFFDh(R/W) SNATCHER
                       (pSltAdr(15 downto 13) = "101" and SccModeB(4) = '1' and Dec1FFE /= '1')
                  ) else '0';

  RamAdrEx  <= '1' when SccModeA(6) = '1' and pSltAdr(14 downto 13) /= "11" else '0';

  -- external port map

  pRamCeX_n <= '0' when SccMemCs = '1' and RamAdrEx = '0' else '1';

  pRamOeX_n <= '0' when pSltRd_n = '0' and pSltRst_n = '1' else '1';
  
  pRamWeX_n <= '0' when pSltWr_n = '0' and pSltRst_n = '1' else '1';

  pRamAdrX  <= SccBank0(5 downto 0) & pSltAdr(12 downto 0) when pSltAdr(14 downto 13) = "10" else
               SccBank1(5 downto 0) & pSltAdr(12 downto 0) when pSltAdr(14 downto 13) = "11" else
               SccBank2(5 downto 0) & pSltAdr(12 downto 0) when pSltAdr(14 downto 13) = "00" else
               SccBank3(5 downto 0) & pSltAdr(12 downto 0);

  pRamDatX  <= pSltDat when pSltWr_n = '0' else (others => 'Z');
    
end RTL;
